@@ -8,15 +8,18 @@
  * able to verify hasn't been messed with afterwards.
  */
 #include "cache.h"
+#include "progress.h"
 #include "csum-file.h"
 
-static void sha1flush(struct sha1file *f, unsigned int count)
+static void flush(struct sha1file *f, unsigned int count)
 {
 	void *buf = f->buffer;
 
 	for (;;) {
 		int ret = xwrite(f->fd, buf, count);
 		if (ret > 0) {
+			f->total += ret;
+			display_throughput(f->tp, f->total);
 			buf = (char *) buf + ret;
 			count -= ret;
 			if (count)
@@ -29,22 +32,38 @@ static void sha1flush(struct sha1file *f, unsigned int count)
 	}
 }
 
-int sha1close(struct sha1file *f, unsigned char *result, int update)
+void sha1flush(struct sha1file *f)
 {
 	unsigned offset = f->offset;
+
 	if (offset) {
 		SHA1_Update(&f->ctx, f->buffer, offset);
-		sha1flush(f, offset);
+		flush(f, offset);
+		f->offset = 0;
 	}
+}
+
+int sha1close(struct sha1file *f, unsigned char *result, unsigned int flags)
+{
+	int fd;
+
+	sha1flush(f);
 	SHA1_Final(f->buffer, &f->ctx);
 	if (result)
 		hashcpy(result, f->buffer);
-	if (update)
-		sha1flush(f, 20);
-	if (close(f->fd))
-		die("%s: sha1 file error on close (%s)", f->name, strerror(errno));
+	if (flags & (CSUM_CLOSE | CSUM_FSYNC)) {
+		/* write checksum and close fd */
+		flush(f, 20);
+		if (flags & CSUM_FSYNC)
+			fsync_or_die(f->fd, f->name);
+		if (close(f->fd))
+			die("%s: sha1 file error on close (%s)",
+			    f->name, strerror(errno));
+		fd = 0;
+	} else
+		fd = f->fd;
 	free(f);
-	return 0;
+	return fd;
 }
 
 int sha1write(struct sha1file *f, void *buf, unsigned int count)
@@ -63,7 +82,7 @@ int sha1write(struct sha1file *f, void *buf, unsigned int count)
 		left -= nr;
 		if (!left) {
 			SHA1_Update(&f->ctx, f->buffer, offset);
-			sha1flush(f, offset);
+			flush(f, offset);
 			offset = 0;
 		}
 		f->offset = offset;
@@ -71,82 +90,24 @@ int sha1write(struct sha1file *f, void *buf, unsigned int count)
 	return 0;
 }
 
-struct sha1file *sha1create(const char *fmt, ...)
-{
-	struct sha1file *f;
-	unsigned len;
-	va_list arg;
-	int fd;
-
-	f = xmalloc(sizeof(*f));
-
-	va_start(arg, fmt);
-	len = vsnprintf(f->name, sizeof(f->name), fmt, arg);
-	va_end(arg);
-	if (len >= PATH_MAX)
-		die("you wascally wabbit, you");
-	f->namelen = len;
-
-	fd = open(f->name, O_CREAT | O_EXCL | O_WRONLY, 0666);
-	if (fd < 0)
-		die("unable to open %s (%s)", f->name, strerror(errno));
-	f->fd = fd;
-	f->error = 0;
-	f->offset = 0;
-	f->do_crc = 0;
-	SHA1_Init(&f->ctx);
-	return f;
-}
-
 struct sha1file *sha1fd(int fd, const char *name)
 {
-	struct sha1file *f;
-	unsigned len;
-
-	f = xmalloc(sizeof(*f));
-
-	len = strlen(name);
-	if (len >= PATH_MAX)
-		die("you wascally wabbit, you");
-	f->namelen = len;
-	memcpy(f->name, name, len+1);
+	return sha1fd_throughput(fd, name, NULL);
+}
 
+struct sha1file *sha1fd_throughput(int fd, const char *name, struct progress *tp)
+{
+	struct sha1file *f = xmalloc(sizeof(*f));
 	f->fd = fd;
-	f->error = 0;
 	f->offset = 0;
+	f->total = 0;
+	f->tp = tp;
+	f->name = name;
 	f->do_crc = 0;
 	SHA1_Init(&f->ctx);
 	return f;
 }
 
-int sha1write_compressed(struct sha1file *f, void *in, unsigned int size)
-{
-	z_stream stream;
-	unsigned long maxsize;
-	void *out;
-
-	memset(&stream, 0, sizeof(stream));
-	deflateInit(&stream, zlib_compression_level);
-	maxsize = deflateBound(&stream, size);
-	out = xmalloc(maxsize);
-
-	/* Compress it */
-	stream.next_in = in;
-	stream.avail_in = size;
-
-	stream.next_out = out;
-	stream.avail_out = maxsize;
-
-	while (deflate(&stream, Z_FINISH) == Z_OK)
-		/* nothing */;
-	deflateEnd(&stream);
-
-	size = stream.total_out;
-	sha1write(f, out, size);
-	free(out);
-	return size;
-}
-
 void crc32_begin(struct sha1file *f)
 {
 	f->crc32 = crc32(0, Z_NULL, 0);
