@@ -44,14 +44,14 @@ do
 	'
 
 	test_expect_success "am$with3 --skip continue after failed am$with3" '
-		test_must_fail git-am$with3 --skip >output &&
+		test_must_fail git am$with3 --skip >output &&
 		test "$(grep "^Applying" output)" = "Applying: 6" &&
 		test_cmp file-2-expect file-2 &&
 		test ! -f .git/rr-cache/MERGE_RR
 	'
 
 	test_expect_success "am --abort goes back after failed am$with3" '
-		git-am --abort &&
+		git am --abort &&
 		git rev-parse HEAD >actual &&
 		git rev-parse initial >expect &&
 		test_cmp expect actual &&
